# GenoVi is a pipeline that generates circular maps for bacterial (complete or non-complete)
# genomes using Circos software. It also allows the user to annotate COG classifications
# through DeepNOG predictions.
# 
# GenoVi is under a BY-NC-SA Creative Commons License, Please cite. Cumsille et al., 2021
# You may remix, tweak, and build upon this work even for commercial purposes, as long as
# you credit this work and license your new creations under the identical terms.
# 
# Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
# For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl.
#
# This file defines an utility for transforming GenBank flat files into .fna FASTA files.
# Intended for GenoVi usage, might not work in other cases.
# Based on Peter Cock pipeline: http://www.warwick.ac.uk/go/peter_cock/python/genbank2fasta/


from Bio import SeqIO
import argparse

__all__ = ['gbkToFna', 'mainFna',
           ]

def gbkToFna(input, output = None, verbose = False):
    if output is None:
        output = input + "_converted.fna"
    try:
        input_handle  = open(input, "r")
    except:
        input_handle  = open(input + ".gbk", "r")
    output_handle = open(output, "w")

    #Short version:
    #SeqIO.write(SeqIO.parse(input_handle, "genbank"), output_handle, "fasta")

    #Long version, allows full control of fasta output
    for seq_record in SeqIO.parse(input_handle, "genbank"):
        if verbose:
            print("Dealing with GenBank record %s" % seq_record.id)
        try: # The old way, removed in Biopython 1.73
            fasta = seq_record.seq.tostring()
        except AttributeError: # The new way, needs Biopython 1.45 or later.
            fasta = str(seq_record.seq)
        
        output_handle.write(">%s %s\n%s\n" % (
            seq_record.id,
            seq_record.description,
            "\n".join([fasta[i:i+60] for i in range(0, len(fasta), 60)])))

    output_handle.close()
    input_handle.close()
    if verbose:
        print("Done")


def mainFna():
    parser = argparse.ArgumentParser(description="Starting from a GenBank flat file, this simplifies it as a .fna")
    parser.add_argument("-i", "--inputFile", help = "flat file to be translated", required = True)
    parser.add_argument("-o", "--outputFile", help = "output file name" )
    args = parser.parse_args()
    if ".gb" in args.inputFile:
        if args.inputFile[-4] == ".":
            args.inputFile = args.inputFile[:-4]
        elif args.inputFile[-3] == ".":
            args.inputFile = args.inputFile[:-3]
    gbkToFna(args.inputFile, args.outputFile)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Starting from a GenBank flat file, this simplifies it as a .fna")
    parser.add_argument("-i", "--inputFile", help = "flat file to be translated", required = True)
    parser.add_argument("-o", "--outputFile", help = "output file name" )
    args = parser.parse_args()
    if ".gb" in args.inputFile:
        if args.inputFile[-4] == ".":
            args.inputFile = args.inputFile[:-4]
        elif args.inputFile[-3] == ".":
            args.inputFile = args.inputFile[:-3]
    gbkToFna(args.inputFile, args.outputFile)



# GenoVi is a pipeline that generates circular maps for bacterial (complete or non-complete)
# genomes using Circos software. It also allows the user to annotate COG classifications
# through DeepNOG predictions.
# 
# GenoVi is under a BY-NC-SA Creative Commons License, Please cite. Cumsille et al., 2021
# You may remix, tweak, and build upon this work even for commercial purposes, as long as
# you credit this work and license your new creations under the identical terms.
# 
# Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
# For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl.
#
# This script generates an image including all the others (Circos-generated SVGs)
# Input: List of dictionaries, with filename and image desired size (relative)
# Example: [{"fileName": "img1.svg", "size": 30000},
#           {"fileName": "img2.svg", "size": 10000}]


from math import sqrt
import re

__all__ = ['mergeImages',
           ]

def mergeImages(images, outFile = "merged.svg", align = "auto", scale = "variable", background_colour = "none", sort = False, captions_position = "normal"):
    if re.match("^\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*$", background_colour):
        background_colour = "rgb(" + background_colour + ")"
    totalWidth = 0
    extraElements = ""
    if sort:
        images.sort(key=lambda x:x["size"], reverse = True)
    for image in images:
        totalWidth += image["size"]
    if scale == "variable":
        totalVariableWidth = 0
        rectangleSize = 600 # Must be in range [0-3000]
        rectIsDrawn = False
        for i in range(len(images)):
            images[i]["scale"] = "fixed"
            if images[i]["size"] < totalWidth/10:
                
                for image in images[i:]:
                    image["scale"] = "variable"
                    totalVariableWidth += image["size"]
                    totalWidth -= image["size"]
                totalWidth += totalWidth * rectangleSize / 3000
                break
    elif scale == "sqrt":
        totalWidth = 0
        for i in range(len(images)):
            images[i]["size"] =  sqrt((images[i]["size"] * images[0]["size"]))
            totalWidth += images[i]["size"]
                
    
    file = open(outFile, "w")
    file.write('''<?xml version="1.0" encoding="utf-8" standalone="no"?>\n<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">\n''')
    if captions_position == "right":
        file.write('''<svg width="4000px" height="3000px" ''')
    elif captions_position == "left":
        file.write('''<svg width="4000px" viewBox="-1000 0 4000 3000" ''')
    elif captions_position == "top" or captions_position == "up":
        file.write('''<svg width="3000px" viewBox="0 -1000 3000 4000" ''')
    elif captions_position == "bottom" or captions_position == "down":
        file.write('''<svg width="3000px" height="4000px" ''')
    else:
        file.write('''<svg width="3000px" height="3000px" ''')
    file.write('''version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n\n''')
    file.write('<rect x="-1000" y="-1000" width="5000px" height="5000px" style="fill:{};"/>'.format(background_colour))
    currentX = 0
    beginGroup = '<g transform="translate({},{}) scale({})">\n'
    if align == "auto":
        if len(images) > 1 and images[1]["size"] > images[0]["size"]*0.75:
            align = "U"
        else:
            align = "A"
    imageIndex = 0
    for image in images:
        inFile = open(image["fileName"])
        for line in inFile:
            if("<svg " in line):
                break
        if align == "center":
            y= 3000 * (totalWidth - image["size"]) / (2 * totalWidth)
            file.write(beginGroup.format(currentX, y, float(image["size"])/totalWidth))
        elif align == "bottom":
            y= 3000 * (totalWidth - image["size"]) / (totalWidth)
            file.write(beginGroup.format(currentX, y, float(image["size"])/totalWidth))
        elif align == "top":
            y= 0
            file.write(beginGroup.format(currentX, y, float(image["size"])/totalWidth))
        elif align == "A":
            if imageIndex == 0:
                firstSize = float(image["size"])/totalWidth
                file.write(beginGroup.format(1500 - (3000*firstSize/2.0), 0, firstSize))
                currentX += 1500 - 1500*(1-firstSize) - 3000*float(image["size"])/totalWidth
                if scale == "variable":
                    currentX -= rectangleSize/2
            else:
                if scale == "variable" and image["scale"] == "variable":
                    if not rectIsDrawn:
                        rectangleSize = (2950 - firstSize*3000) * totalVariableWidth / max([x["size"] for x in images[imageIndex:]])
                        extraElements = '''<rect x="{0}" y="{1}" width="{2}" height="{3}" fill="none" stroke = "black"/>'''.format(currentX, 3000*firstSize, rectangleSize, 35 + max([x["size"] for x in images[imageIndex:]])*(rectangleSize*11/12)/totalVariableWidth)
                        extraElements += '<text x="{0}" y="{1}" font-size="30" font-family="CMUBright-Roman" text-anchor="left">scale: x{2}</text>\n'.format(currentX, 3000*firstSize + max([x["size"] for x in images[imageIndex:]])*(rectangleSize*11/12)/totalVariableWidth + 30, round(totalWidth/((3000/rectangleSize)*totalVariableWidth), 1))
                        rectIsDrawn = True
                    file.write(beginGroup.format(currentX, 3000*firstSize, float(image["size"])/((3000/rectangleSize)*totalVariableWidth)))
                    image["size"] = image["size"]*totalWidth/((3000/rectangleSize)*totalVariableWidth)
                else:
                    file.write(beginGroup.format(currentX, 3000*firstSize, float(image["size"])/totalWidth))
        elif align == "<":
            if currentX == 0:
                firstSize = float(image["size"])/totalWidth
                file.write(beginGroup.format(0, 1500 - (3000*firstSize/2.0), firstSize))
                currentX += 1500 - 1500*(1-firstSize) - 3000*float(image["size"])/totalWidth
                if scale == "variable":
                    currentX -= rectangleSize/2
            else:
                if scale == "variable" and image["scale"] == "variable":
                    if not rectIsDrawn:
                        extraElements = '<rect x="{0}" y="{1}" width="{2}" height="{3}" fill="none" stroke = "black"/>'.format(3000*firstSize, currentX, 35 + max([x["size"] for x in images[imageIndex:]])*(rectangleSize*11/12)/totalVariableWidth, rectangleSize)
                        extraElements += '<text x="{0}" y="{1}" font-size="30" font-family="CMUBright-Roman" text-anchor="left">scale: x{2}</text>\n'.format(3000*firstSize,currentX + (rectangleSize + 35), round(totalWidth/((3000/rectangleSize)*totalVariableWidth)))
                        rectIsDrawn = True
                    file.write(beginGroup.format(3000*firstSize, currentX, float(image["size"])/((3000/rectangleSize)*totalVariableWidth)))
                    image["size"] = image["size"]*totalWidth/((3000/rectangleSize)*totalVariableWidth)
                else:
                    file.write(beginGroup.format(3000*firstSize, currentX, float(image["size"])/totalWidth))
        elif align == "U":
            if imageIndex == 0:
                firstSize = float(image["size"])/totalWidth
                file.write(beginGroup.format(1500 - (3000*firstSize), 0, firstSize))
                currentX = -1 - 3000*float(image["size"])/totalWidth
            elif imageIndex == 1:
                secondSize = float(image["size"])/totalWidth
                file.write(beginGroup.format(1500, 0, secondSize))
                currentX = 1500 + 1500*(firstSize + secondSize - 1) - 3000*float(image["size"])/totalWidth
                if scale == "variable":
                    currentX -= rectangleSize/2
            else:
                if scale == "variable" and image["scale"] == "variable":
                    if not rectIsDrawn:
                        extraElements = '<rect x="{0}" y="{1}" width="{2}" height="{3}" fill="none" stroke = "black"/>'.format(currentX, 3000*max([firstSize, secondSize]), rectangleSize, 35 + max([x["size"] for x in images[imageIndex:]])*(rectangleSize*11/12)/totalVariableWidth)
                        extraElements += '<text x="{0}" y="{1}" font-size="30" font-family="CMUBright-Roman" text-anchor="left">scale: x{2}</text>\n'.format(currentX, 3000*max([firstSize, secondSize]) + max([x["size"] for x in images[imageIndex:]])*(rectangleSize*11/12)/totalVariableWidth + 30, round(totalWidth/((3000/rectangleSize)*totalVariableWidth)))
                        rectIsDrawn = True
                    file.write(beginGroup.format(currentX, 3000*max([firstSize, secondSize]), float(image["size"])/((3000/rectangleSize)*totalVariableWidth)))
                    image["size"] = image["size"]*totalWidth/((3000/rectangleSize)*totalVariableWidth)
                else:
                    file.write(beginGroup.format(currentX, 3000*max([firstSize, secondSize]), float(image["size"])/totalWidth))
        for line in inFile:
            if("</svg>" in line):
                break
            file.write(line)
        file.write('</g>\n')
        inFile.close()
        currentX += 3000*float(image["size"])/totalWidth
        imageIndex += 1
    file.write(extraElements)
    file.write('</svg>')
    file.close()
    



import numpy as np
import csv
import matplotlib.pyplot as plt


def fill_unique_chrms(chrms, n):
	
	def check(lst, n):
		if len(lst[0]) == n:
			return list(map(int,lst[1]))
		new_vals = [0]*n
		for i in range(n):
			if str(i+1) in lst[0]:
				new_vals[i] = lst[1][np.where(lst[0] == str(i+1))[0]][0]
			else:
				continue
		return list(map(int,new_vals))
	
	pos = check(chrms[0],n)
	neg = check(chrms[1],n)
	
	return [sum(x) for x in zip(pos, neg)]
		

def gral_table(lengths, contents, chrms, output):
	header = ["Replicon", "Size (bp)", "GC content (%)", "CDS", "tRNA", "rRNA"]
	ttrnas = fill_unique_chrms(chrms[0], len(lengths))
	rrnas = fill_unique_chrms(chrms[1], len(lengths))
	cdss = fill_unique_chrms(chrms[2], len(lengths))
	
	csv_file = open(output, "w")
	writer = csv.writer(csv_file)
	writer.writerow(header)
	
	for i in range(len(lengths)):
		avg = contents[i][0]/contents[i][1]
		line = map(str, [i+1, lengths[i], avg, cdss[i], ttrnas[i], rrnas[i]])
		writer.writerow(line)
		
	avg_total = np.sum([x[0] for x in contents])/np.sum([x[1] for x in contents])
	footer = map(str, ["Total", np.sum(lengths), avg_total, np.sum(cdss), np.sum(ttrnas), np.sum(rrnas)])
	
	writer.writerow(footer)
	csv_file.close()
	
	return
	
def cogs_classif(hist, output):
	
	header1 = ["","Cellular Processes and Signaling"]+[""]*9+["Information Storage and Processing"]+[""]*5+["Metabolism"]+[""]*7+["Poorly Characterized","","",""]
	header2 = ["Replicon","D","M","N","O","T","U","V","W","Y","Z","A","B","J","K","L","X","C","E","F","G","H","I","P","Q","R","S","Unclassified"]
	
	csv_file = open(output, 'w')
	writer = csv.writer(csv_file)
	writer.writerow(header1)
	writer.writerow(header2)
	
	for i in range(len(hist.columns) - 2):
		line = map(str,[i+1] + [hist[hist["COG Category"] == c]["chr"+str(i+1)].item() for c in header2[1:]])
		writer.writerow(line)
		
	footer = map(str,["Total"] + [hist[hist["COG Category"] == c]["Frequency"].item() for c in header2[1:]])
	writer.writerow(footer)
	csv_file.close()
	
	return
	
def draw_histogram(hist, output):
	colors = ["#697BB7", '#304E9D', '#6A9AB5', '#1D82B0', '#4CA9B5', '#1C6875', '#6DBFA4', '#1A936F', '#4EB160', '#1C7633', '#E34949', '#CD1B1B', '#AD5B9F', '#A3378C', '#8F76B4',
	          '#533D91', '#B5D27B', '#83AD29', '#BCB868', '#988F1C', '#EDC28A', '#B7822A', '#DD8950', '#C65F17', '#696969', '#999999', '#EAEAEA']
	ax = hist.plot.bar(x="COG Category", y="Frequency", rot=90, legend=False, color=colors)
	for p in ax.patches:
		ax.annotate(str(p.get_height()), (p.get_x()+p.get_width()/2, p.get_height()+5), ha='center', va='center')
	plt.tight_layout()
	ax.figure.savefig(output+"_COG_Histogram.png", dpi=150)
	cogs_classif(hist, output+"_COG_Classification.csv")
	
	return



# GenoVi is a pipeline that generates circular maps for bacterial (complete or non-complete)
# genomes using Circos software. It also allows the user to annotate COG classifications
# through DeepNOG predictions.
# 
# GenoVi is under a BY-NC-SA Creative Commons License, Please cite. Cumsille et al., 2021
# You may remix, tweak, and build upon this work even for commercial purposes, as long as
# you credit this work and license your new creations under the identical terms.
# 
# Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
# For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl


import re

__all__ = ['addText',
           ]

# Adds text elements to Circos-generated SVG image.
# May include title, contig size and colour legend.
def addText(text, position = "center", inFile="circos.svg", outFile="default", italic=2,
            captions=True, cogs_captions=True, captionsPosition = "botom-right", cogs = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"},
            pCDS_colour = "180, 205, 222", nCDS_colour = "150, 200, 150", tRNA_colour = "150, 5, 50", rRNA_colour = "150, 150, 50", GC_content_colour = "23, 0, 115", size = "", font_colour = "0, 0, 0"):
    if re.match("^\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*$", font_colour):
        font_colour = "rgb(" + font_colour + ")"
    if(outFile == "default"):
        outFile = "titled_" + inFile
    if text != "":
        textSize = 1500/len(text)
        if(position == "center"):
            verticalPosition = "1500"
        elif(position == "top"):
            verticalPosition = "150"
        elif(position == "bottom"):
            verticalPosition = "2900"
        if italic==0:
            textElement = '<text x="1500" y="{0}" font-size="{1}" font-family="CMUBright-Roman" text-anchor="middle" fill="{3}">{2}</text>\n'.format(verticalPosition, textSize, text, font_colour)
        else:
            textList = re.split(r"\s+", text)
            if len(textList) > italic:
                italicText = " ".join(textList[:italic])
                nonItalicText = " ".join(textList[italic:])
                textElement = '<text x="1500" y="{0}" font-size="{1}" font-family="CMUBright-Roman" text-anchor="middle" fill="{4}"><tspan font-style="italic">{2} </tspan>{3}</text>\n'.format(verticalPosition, textSize, italicText, nonItalicText, font_colour)
            else:
                textElement = '<text x="1500" y="{0}" font-size="{1}" font-family="CMUBright-Roman" text-anchor="middle" font-style="italic" fill = "{3}">{2}</text>\n'.format(verticalPosition, textSize, text, font_colour)
    if size != "":
        if size > 1000000:
            size = str(round(size / 1000000, 2)) + " Mb"
        elif size > 1000:
            size = str(round(size / 1000, 2)) + " kb"
        sizeElement = '<text x="1500" y="1560" font-size="64" font-family="CMUBright-Roman" text-anchor="middle" fill="{1}">{0}</text>\n'.format(size, font_colour)
    source = open(inFile)
    destiny = open(outFile, "w")
    background_has_been_amplified = False
    for line in source:
        if captionsPosition == "right" and '<svg width="3000px" height="3000px"' in line:
            destiny.write('<svg width="4000px" height="3000px" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n')
            continue
        elif captionsPosition == "left" and '<svg width="3000px" height="3000px"' in line:
            destiny.write('<svg viewBox="-1000 0 4000 3000" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n')
            continue
        elif captionsPosition == "bottom" and '<svg width="3000px" height="3000px"' in line:
            destiny.write('<svg viewBox="0 0 3000 4000" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n')
            continue
        elif captionsPosition == "top" and '<svg width="3000px" height="3000px"' in line:
            destiny.write('<svg viewBox="0 -1000 3000 4000" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n')
            continue
        elif not background_has_been_amplified and (captionsPosition == "right" or captionsPosition == "left" or captionsPosition == "top" or captionsPosition == "up" or captionsPosition == "bottom" or captionsPosition == "down") and re.match('<rect x="-?1?0?0?0" y="0" width="[34]000px" height="3000px" style="fill:(.*);"/>',line):
            fill = re.match('<rect x="-?1?0?0?0" y="-?1?0?0?0" width="[34]000px" height="[34]000px" style="fill:(.*);"/>',line).groups()[0]
            destiny.write('<rect x="-1000" y="-1000" width="5000px" height="5000px" style="fill:{};"/>\n'.format(fill))
            background_has_been_amplified = True
            continue
        if('</svg>' in line):
            if text != "":
                destiny.write(textElement)
            if size != "":
                destiny.write(sizeElement)
            if captions:
                legendElement = '<g>'
                index = 0
                legendPiece = '''<rect x="{4}" y="{0}" width="15" height="15" fill="rgb({3})" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="16.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                otherLegendPiece = '''<path d="M{4} {0} m0 10 h3 l2 -5 5 10 2-5 3 0" fill="{3}" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="16.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''

                features = [{"description":"positive CDS", "colour": pCDS_colour},
                    {"description":"negative CDS", "colour": nCDS_colour},
                    {"description":"tRNA", "colour": tRNA_colour},
                    {"description":"rRNA", "colour": rRNA_colour},
                    {"description":"GC content", "colour": GC_content_colour},
                    {"description":"GC skew", "colour": "line"}]
                if cogs_captions:
                    COGs1 = [{"symbol": "D", "description":"[D] Cell cycle control, adivision, chromosome partitioning", "colour": "105, 123, 183"},
                    {"symbol": "M", "description":"[M] Cell wall/membrane/envelope biogenesis", "colour": "48, 78, 157"},
                    {"symbol": "N", "description":"[N] Cell motility", "colour": "106, 154, 181"},
                    {"symbol": "O", "description":"[O] Post translational modification, protein turnover, chaperones", "colour": "29, 130, 176"},
                    {"symbol": "T", "description":"[T] Signal transduction mechanism", "colour": "76, 169, 181"},
                    {"symbol": "U", "description":"[U] Intracellular trafficking, secretion and vesicular transport", "colour": "28, 104, 117"},
                    {"symbol": "V", "description":"[V] Defense mechanism", "colour": "109, 191, 164"},
                    {"symbol": "W", "description":"[W] Extracellular structures", "colour": "26, 147, 111"},
                    {"symbol": "Y", "description":"[Y] Nuclear structure", "colour": "78, 177, 96"},
                    {"symbol": "Z", "description":"[Z] Cytoskeleton", "colour": "28, 118, 51"}]
                    
                    COGs2 = [{"symbol": "A", "description":"[A] RNA processing and modification", "colour": "227, 73, 73"},
                    {"symbol": "B", "description":"[B] Chromatin structure and dynamics", "colour": "205, 27, 27"},
                    {"symbol": "J", "description":"[J] Translation, ribosomal structure and biogenesis", "colour": "173, 91, 159"},
                    {"symbol": "K", "description":"[K] Transcription", "colour": "163, 55, 140"},
                    {"symbol": "L", "description":"[L] Replication, recombination and repair", "colour": "143, 118, 180"},
                    {"symbol": "X", "description":"[X] Mobilome: prophages, transposons", "colour": "83, 61, 145"},
                    {"symbol": "space"},
                    {"symbol": "C", "description":"[C] Energy production and conversion", "colour": "181, 210, 123"},
                    {"symbol": "E", "description":"[E] Amino acid transport and metabolism", "colour": "131, 173, 41"},
                    {"symbol": "F", "description":"[F] Nucleotide transport and metabolism", "colour": "188, 184, 104"},
                    {"symbol": "G", "description":"[G] Carbohydrate transport and metabolism", "colour": "152, 143, 28"},
                    {"symbol": "H", "description":"[H] Coenzyme transport and metabolism", "colour": "237, 194, 138"},
                    {"symbol": "I", "description":"[I] Lipid transport and metabolism", "colour": "183, 130, 42"},
                    {"symbol": "P", "description":"[P] Inorganic ion transport and metabolism", "colour": "221, 137, 80"},
                    {"symbol": "Q", "description":"[Q] Secondary metabolities biosynthesis, transport and metabolism", "colour": "198, 95, 23"},
                    {"symbol": "space"},
                    {"symbol": "R", "description":"[R] General function prediction only", "colour": "105, 105, 105"},
                    {"symbol": "S", "description":"[S] Function unknown", "colour": "153, 153, 153"},
                    {"symbol": "None", "description":"Unclassified", "colour": "234, 234, 234"}]
                    
                    if captionsPosition == "right" or captionsPosition == "left":
                        legendPiece = '''<rect x="{4}" y="{0}" width="15" height="15" fill="rgb({3})" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        otherLegendPiece = '''<path d="M{4} {0} m0 10 h3 l2 -5 5 10 2-5 3 0" fill="{3}" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        index = 0
                        for COG in [*COGs1, *COGs2]:
                            
                            if COG["symbol"] in cogs:
                                legendElement += legendPiece.format(50 + index*35, 64 + index*35, COG["description"], COG["colour"],
                                                 3000 if captionsPosition == "right" else -900, 3030 if captionsPosition == "right" else -870, font_colour)
                                index += 1
                            elif COG["symbol"] == "space":
                                index += 1
                        index += 1
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            legendElement += legendPiece.format(50 + index*35, 64 + index*35, feature["description"], feature["colour"],
                                             3000 if captionsPosition == "right" else -900, 3030 if captionsPosition == "right" else -870, font_colour)
                            index += 1
                    elif captionsPosition == "top" or captionsPosition == "bottom":
                        legendPiece = '''<rect x="{4}" y="{0}" width="30" height="30" fill="rgb({3})" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="38.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        otherLegendPiece = '''<path d="M{4} {0} m0 20 h6 l4 -10 10 20 4-10 6 0" fill="{3}" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="40.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        index = 0
                        for COG in COGs1:
                            
                            if COG["symbol"] in cogs:
                                legendElement += legendPiece.format((-900 if captionsPosition == "top" else 3000) + index*50, (-870 if captionsPosition == "top" else 3030) + index*50, COG["description"], COG["colour"],
                                                 40, 80, font_colour)
                                index += 1
                            elif COG["symbol"] == "space":
                                index += 1
                        index = 0
                        for COG in COGs2:
                            
                            if COG["symbol"] in cogs:
                                legendElement += legendPiece.format((-900 if captionsPosition == "top" else 3000) + index*50, (-870 if captionsPosition == "top" else 3030) + index*50, COG["description"], COG["colour"],
                                                 1340, 1380, font_colour)
                                index += 1
                            elif COG["symbol"] == "space":
                                index += 1
                        index = 0
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            legendElement += legendPiece.format((-900 if captionsPosition == "top" else 3000) + index*50, (-870 if captionsPosition == "top" else 3030) + index*50, feature["description"], feature["colour"],
                                             2640, 2680, font_colour)
                            index += 1
                    else:
                        for COG in COGs1[::-1]:
                            if COG["symbol"] in cogs:
                                if captionsPosition == "top-right":
                                    legendElement += legendPiece.format(50 + index*25, 62 + index*25, COG["description"], COG["colour"], 1800, 1830, font_colour)
                                elif captionsPosition == "bottom-right":
                                    legendElement += legendPiece.format(2980 - index*25, 2992 - index*25, COG["description"], COG["colour"], 1800, 1830, font_colour)
                                elif captionsPosition == "top-left":
                                    legendElement += legendPiece.format(50 + index*25, 62 + index*25, COG["description"], COG["colour"], 650, 680, font_colour)
                                else:
                                    legendElement += legendPiece.format(2980 - index*25, 2992 - index*25, COG["description"], COG["colour"], 650, 680, font_colour)
                                index += 1
                        index = 0
                        for COG in COGs2[::-1]:
                            if COG["symbol"] in cogs:
                                if captionsPosition == "top-right":
                                    legendElement += legendPiece.format(50 + index*25, 62 + index*25, COG["description"], COG["colour"], 2400, 2430, font_colour)
                                elif captionsPosition == "bottom-right":
                                    legendElement += legendPiece.format(2980 - index*25, 2992 - index*25, COG["description"], COG["colour"], 2400, 2430, font_colour)
                                elif captionsPosition == "top-left":
                                    legendElement += legendPiece.format(50 + index*25, 62 + index*25, COG["description"], COG["colour"], 50, 80, font_colour)
                                else:
                                    legendElement += legendPiece.format(2980 - index*25, 2992 - index*25, COG["description"], COG["colour"], 50, 80, font_colour)
                                index += 1
                            elif COG["symbol"] == "space":
                                index += 1
                        index = 0
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            if captionsPosition == "top-right":
                                legendElement += legendPiece.format(50 + index*25, 62 + index*25, feature["description"], feature["colour"], 1600, 1630, font_colour)
                            elif captionsPosition == "bottom-right":
                                legendElement += legendPiece.format(2855 + index*25, 2862 + index*25, feature["description"], feature["colour"], 1600, 1630, font_colour)
                            elif captionsPosition == "top-left":
                                legendElement += legendPiece.format(50 + index*25, 62 + index*25, feature["description"], feature["colour"], 450, 480, font_colour)
                            else:
                                legendElement += legendPiece.format(2855 + index*25, 2862 + index*25, feature["description"], feature["colour"], 450, 480, font_colour)
                            index += 1
                else: # if not cogs_legend
                    index = 0
                    if captionsPosition == "right":
                        legendPiece = '''<rect x="{4}" y="{0}" width="15" height="15" fill="rgb({3})" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        otherLegendPiece = '''<path d="M{4} {0} m0 10 h3 l2 -5 5 10 2-5 3 0" fill="{3}" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            legendElement += legendPiece.format(50 + index*35, 64 + index*35, feature["description"], feature["colour"], 3000, 3030, font_colour)
                            index += 1
                    elif captionsPosition == "left":
                        legendPiece = '''<rect x="{4}" y="{0}" width="15" height="15" fill="rgb({3})" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        otherLegendPiece = '''<path d="M{4} {0} m0 10 h3 l2 -5 5 10 2-5 3 0" fill="{3}" stroke="{6}"/>
                                <text x="{5}" y="{1}" font-size="25.0px" font-family="CMUBright-Roman" style="text-anchor:start;" fill="{6}">{2}</text>\n'''
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            legendElement += legendPiece.format(50 + index*35, 64 + index*35, feature["description"], feature["colour"], -900, -870, font_colour)
                            index += 1
                    else:
                        for feature in features:
                            if feature["colour"] == "line":
                                legendPiece = otherLegendPiece
                                feature["colour"] = "none"
                            if "eval" in feature["colour"]:
                                feature["colour"] = "100,100,100"
                            if captionsPosition == "top-right":
                                legendElement += legendPiece.format(50 + index*25, 68 + index*25, feature["description"], feature["colour"], 2700, 2730, font_colour)
                            elif captionsPosition == "bottom-right":
                                legendElement += legendPiece.format(2830 + index*25, 2848 + index*25, feature["description"], feature["colour"], 2700, 2730, font_colour)
                            elif captionsPosition == "top-left":
                                legendElement += legendPiece.format(50 + index*25, 68 + index*25, feature["description"], feature["colour"], 50, 80, font_colour)
                            else:
                                legendElement += legendPiece.format(2830 + index*25, 2848 + index*25, feature["description"], feature["colour"], 50, 80, font_colour)
                            index += 1
                legendElement += '</g>\n'
                destiny.write(legendElement)
        destiny.write(line)
    destiny.close()
    source.close()



# GenoVi is a pipeline that generates circular maps for bacterial (complete or non-complete)
# genomes using Circos software. It also allows the user to annotate COG classifications
# through DeepNOG predictions.
# 
# GenoVi is under a BY-NC-SA Creative Commons License, Please cite. Cumsille et al., 2021
# You may remix, tweak, and build upon this work even for commercial purposes, as long as
# you credit this work and license your new creations under the identical terms.
# 
# Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
# For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl

import re

__all__ = ['parseColours']

# This function is for parsing colour schemes
def parseColours(colour_scheme = "auto", background_colour = "none", GC_content = "auto", GC_skew ='auto', tRNA = 'auto', rRNA = 'auto', CDS_positive = 'auto', CDS_negative = 'auto', skew_line_colour = '0, 0, 0'):
    colour_scheme = colour_scheme.lower()
    # if re.match("^\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*,\s*[012]?\d?\d\s*$", background_colour):
    #     background_colour = "rgb(" + background_colour + ")"
    gc_skew_original_argument = re.match("^\s*(?P<red1>[012]?\d?\d)\s*,\s*(?P<green1>[012]?\d?\d)\s*,\s*(?P<blue1>[012]?\d?\d)\s*[,;-]\s*(?P<red2>[012]?\d?\d)\s*,\s*(?P<green2>[012]?\d?\d)\s*,\s*(?P<blue2>[012]?\d?\d)\s*$", GC_skew)
    if gc_skew_original_argument:
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,{},{}),remap_int(var(value),0,0,{},{}),remap_int(var(value),0,0,{},{})))'.format(
            gc_skew_original_argument.group("red1"),
            gc_skew_original_argument.group("red2"),
            gc_skew_original_argument.group("green1"),
            gc_skew_original_argument.group("green2"),
            gc_skew_original_argument.group("blue1"),
            gc_skew_original_argument.group("blue2"))
    
    if colour_scheme == "blue" or colour_scheme == "blues":
        GC_content = "14, 29, 130" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("rdbu-7-div-%d",remap_int(var(value),0,0,7,5)))' if GC_skew == "auto" else GC_skew
        tRNA = "99, 103, 186" if tRNA == "auto" else tRNA
        rRNA = "89, 123, 186" if rRNA == "auto" else rRNA
        CDS_positive = "191, 204, 217" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "171, 178, 217" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "163, 191, 217" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "purple" or colour_scheme == "purples":
        GC_content = "57, 1, 120" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("puor-7-div-%d",remap_int(var(value),0,0,7,5)))' if GC_skew == "auto" else GC_skew
        tRNA = "108, 95, 156" if tRNA == "auto" else tRNA
        rRNA = "128, 85, 156" if rRNA == "auto" else rRNA
        CDS_positive = "196, 187, 237" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "151, 143, 191" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "163, 163, 217" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "soil":
        GC_content = "89, 60, 6" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("brbg-7-div-%d",remap_int(var(value),0,0,7,5)))' if GC_skew == "auto" else GC_skew
        tRNA = "0, 112, 11" if tRNA == "auto" else tRNA
        rRNA = "20, 112, 1" if rRNA == "auto" else rRNA
        CDS_positive = "185, 199, 186" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "175, 201, 178" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "89, 194, 115" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "greyscale" or colour_scheme == "grayscale" or colour_scheme == "grey" or colour_scheme == "gray":
        GC_content = "87, 87, 87" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("rdgy-7-div-%d",remap_int(var(value),0,0,7,5)))' if GC_skew == "auto" else GC_skew
        tRNA = "115, 115, 115" if tRNA == "auto" else tRNA
        rRNA = "115, 115, 115" if rRNA == "auto" else rRNA
        CDS_positive = "209, 209, 209" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "184, 184, 184" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "171, 171, 171" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "velvet" or colour_scheme == "pink" or colour_scheme == "red":
        GC_content = "82, 1, 30" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("purd-7-seq-%d",remap_int(var(value),0,0,7,4)))' if GC_skew == "auto" else GC_skew
        tRNA = "130, 1, 49" if tRNA == "auto" else tRNA
        rRNA = "130, 21, 69" if rRNA == "auto" else rRNA
        CDS_positive = "217, 195, 205" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "196, 187, 193" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "230, 200, 211" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "pastel":
        GC_content = "196, 227, 255" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("pastel1-7-qual-%d",remap_int(var(value),0,0,4,3)))' if GC_skew == "auto" else GC_skew
        tRNA = "143, 143, 143" if tRNA == "auto" else tRNA
        rRNA = "143, 143, 143" if rRNA == "auto" else rRNA
        CDS_positive = "255, 222, 235" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "181, 255, 214" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "150, 150, 150" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "ocean" or colour_scheme == "sea":
        GC_content = "24,138,141" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,127",remap_int(var(value),0,0,23,63),remap_int(var(value),0,0,87,159)))' if GC_skew == "auto" else GC_skew
        tRNA = "0, 83, 83" if tRNA == "auto" else tRNA
        rRNA = "0, 83, 83" if rRNA == "auto" else rRNA
        CDS_positive = "96,221,142" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "23,87,126" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "24,138,141" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "wood":
        GC_content = "150,90,49" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,120,163),remap_int(var(value),0,0,118,124),remap_int(var(value),0,0,37,81)))' if GC_skew == "auto" else GC_skew
        tRNA = "131,62,32" if tRNA == "auto" else tRNA
        rRNA = "62,131,32" if rRNA == "auto" else rRNA
        CDS_positive = "73,115,54" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "185,157,103" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "143,76,40" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "beach":
        GC_content = "231,207,133" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,93,210),remap_int(var(value),0,0,204,173),remap_int(var(value),0,0,167,128)))' if GC_skew == "auto" else GC_skew
        tRNA = "184,141,117" if tRNA == "auto" else tRNA
        rRNA = "117,167,184" if rRNA == "auto" else rRNA
        CDS_positive = "131,246,191" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "79,176,159" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "93,204,167" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "desert":
        GC_content = "225,168,111" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,237,193),remap_int(var(value),0,0,242,98),remap_int(var(value),0,0,154,2)))' if GC_skew == "auto" else GC_skew
        tRNA = "36,16,16" if tRNA == "auto" else tRNA
        rRNA = "180,165,150" if rRNA == "auto" else rRNA
        CDS_positive = "240,198,70" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "237,242,154" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "225,168,111" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "ice" or colour_scheme == "frozen":
        GC_content = "186,242,239" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,37,162),remap_int(var(value),0,0,124,210),remap_int(var(value),0,0,163,223)))' if GC_skew == "auto" else GC_skew
        tRNA = "57,109,124" if tRNA == "auto" else tRNA
        rRNA = "97,129,144" if rRNA == "auto" else rRNA
        CDS_positive = "120,143,255" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "200,223,235" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "162,210,223" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "island":
        GC_content = "75,157,19" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,78,245),remap_int(var(value),0,0,247,229),remap_int(var(value),0,0,171,147)))' if GC_skew == "auto" else GC_skew
        tRNA = "75,157,19" if tRNA == "auto" else tRNA
        rRNA = "180,165,150" if rRNA == "auto" else rRNA
        CDS_positive = "254,155,227" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "154,215,173" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "155,187,89" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "forest":
        GC_content = "45,158,48" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,32,136),remap_int(var(value),0,0,103,208),remap_int(var(value),0,0,78,95)))' if GC_skew == "auto" else GC_skew
        tRNA = "109,137,75" if tRNA == "auto" else tRNA
        rRNA = "104,37,11" if rRNA == "auto" else rRNA
        CDS_positive = "136,208,95" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "105,184,128" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "45,158,48" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "toxic":
        GC_content = "137,4,177" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,106,11),remap_int(var(value),0,0,8,97),remap_int(var(value),0,0,136,11)))' if GC_skew == "auto" else GC_skew
        tRNA = "0,0,0" if tRNA == "auto" else tRNA
        rRNA = "50,50,50" if rRNA == "auto" else rRNA
        CDS_positive = "1,223,1" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "237,4,177" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "137,4,177" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "fire":
        GC_content = "216,0,0" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,0",remap_int(var(value),0,0,216,242),remap_int(var(value),0,0,0,85)))' if GC_skew == "auto" else GC_skew
        tRNA = "140,40,0" if tRNA == "auto" else tRNA
        rRNA = "161,0,0" if rRNA == "auto" else rRNA
        CDS_positive = "255,129,0" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "242,85,0" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "234,35,0" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "spring":
        GC_content = "249,150,174" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,217,107),remap_int(var(value),0,0,182,206),remap_int(var(value),0,0,253,238)))' if GC_skew == "auto" else GC_skew
        tRNA = "137,4,177" if tRNA == "auto" else tRNA
        rRNA = "237,164,207" if rRNA == "auto" else rRNA
        CDS_positive = "186,240,163" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "246,240,163" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "159,244,223" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "electric":
        GC_content = "22,48,190" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,255,160),remap_int(var(value),0,0,255,220),remap_int(var(value),0,0,0,205)))' if GC_skew == "auto" else GC_skew
        tRNA = "130,130,140" if tRNA == "auto" else tRNA
        rRNA = "2,4,100" if rRNA == "auto" else rRNA
        CDS_positive = "220,220,230" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "255,255,155" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "159,244,223" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "stone":
        GC_content = 'eval(sprintf("hsv(200,0.38,0.%d)", 56+rand(10)))' if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,159,45),remap_int(var(value),0,0,181,39),remap_int(var(value),0,0,184,39)))' if GC_skew == "auto" else GC_skew
        tRNA = "60,48,48" if tRNA == "auto" else tRNA
        rRNA = "45,39,39" if rRNA == "auto" else rRNA
        CDS_positive = "159,181,184" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "184,169,159" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "100,100,100" if skew_line_colour == "auto" else skew_line_colour
    elif colour_scheme == "skin":
        GC_content = '224,172,105' if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("%d,%d,%d",remap_int(var(value),0,0,241,141),remap_int(var(value),0,0,194,85),remap_int(var(value),0,0,125,36)))' if GC_skew == "auto" else GC_skew
        tRNA = "141,85,36" if tRNA == "auto" else tRNA
        rRNA = "141,95,66" if rRNA == "auto" else rRNA
        CDS_positive = "255,219,172" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "241,194,125" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "224,172,105" if skew_line_colour == "auto" else skew_line_colour
    else: # auto or neutral
        GC_content = "94, 120, 145" if GC_content == "auto" else GC_content
        GC_skew = 'eval(sprintf("bupu-7-seq-%d",remap_int(var(value),0,0,4,3)))' if GC_skew == "auto" else GC_skew
        tRNA = "67, 14, 110" if tRNA == "auto" else tRNA
        rRNA = "67, 110, 14" if rRNA == "auto" else rRNA
        CDS_positive = "186, 186, 186" if CDS_positive == "auto" else CDS_positive
        CDS_negative = "140, 140, 140" if CDS_negative == "auto" else CDS_negative
        skew_line_colour = "171, 171, 171" if skew_line_colour == "auto" else skew_line_colour
    
    return colour_scheme, background_colour, GC_content, GC_skew, tRNA, rRNA, CDS_positive, CDS_negative, skew_line_colour



# GenoVi is a pipeline that generates circular maps for bacterial (complete or non-complete)
# genomes using Circos software. It also allows the user to annotate COG classifications
# through DeepNOG predictions.
# 
# GenoVi is under a BY-NC-SA Creative Commons License, Please cite. Cumsille et al., 2021
# You may remix, tweak, and build upon this work even for commercial purposes, as long as
# you credit this work and license your new creations under the identical terms.
# 
# Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
# For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl.


import argparse as ap
import os

__all__ = ['create_conf', 'create_conf_main' 
           ]


# Function that writes base CIRCOS configuration files
# circos.conf, highlight.conf, colors_fonts_patterns.conf, housekeeping.conf, image.conf, and ticks.conf.
def create_conf(output,
                folder,
                maxmins,
                font_color = "0, 0, 0",
                GC_content_color = "23, 0, 115",
                GC_skew_color = 'eval(sprintf("rdbu-7-div-%d",remap_int(var(value),0,0,7,5)))',
                CDS_positive_color = '180, 205, 222',
                CDS_negative_color = '53, 176, 42',
                tRNA_color = '150, 5, 50',
                rRNA_color = '150, 150, 50',
                GC_skew_line_color = '0, 0, 0',
                background_color = "transparent",
                cogs = True,
                cogs_p = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "None"},
                cogs_n = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "None"}):
    file = open("circos.conf", "w")
    file.write('''karyotype = {folder}/{output}_bands.kar
chromosomes_units = 100
chromosomes_display_default = yes

<<include conf/colors_fonts_patterns.conf>>

<<include conf/housekeeping.conf>>

<<include conf/highlight.conf>>

<<include conf/ticks.conf>>

# IMAGE
<image>
<<include conf/image.conf>>
</image>

# IDEOGRAM
<ideogram>
<spacing>
default = 0.001r
</spacing>
radius           = 0.8r
thickness        = 40p
fill             = yes
stroke_color     = {font_color}
stroke_thickness = 2p
show_label       = no
label_radius     = dims(image,radius) - 250p
label_size       = 30p
label_parallel   = no
show_bands = yes
fill_bands = yes

</ideogram>

# PLOTS
<plots>

#Plot GC content
<plot>
type       = line
extend_bin = no
color      = {font_color}
fill_under = yes
thickness  = 1
file = {folder}/{output}_GC_content.wig
r0   = 0.62r
r1   = 0.7r
min  = {GC_min}
max  = {GC_max}
<rules>
<rule>
condition = 1
fill_color = {content_fill}
</rule>
</rules>
</plot>

#Plot GC skew
<plot>
type       = line
extend_bin = no
color      = {skew_line}
fill_under = no
thickness  = 0.5
file = {folder}/{output}_GC_skew.wig
r0   = 0.3r
r1   = 0.6r
min  = {skew_min}
max  = {skew_max}
<rules>
<rule>
condition = 1
fill_color = {skew_fill}
</rule>
</rules>
</plot>

# BACKGROUND
background                  = yes
background_stroke_color     = {font_color}
background_stroke_thickness = 1

</plots>
'''.format( folder = folder,
            output = output,
            GC_min = maxmins["min_GC_content"],
            GC_max = maxmins["max_GC_content"],
            skew_min = maxmins["min_skew"],
            skew_max = maxmins["max_skew"],
            content_fill = GC_content_color,
            skew_fill = GC_skew_color,
            skew_line = GC_skew_line_color,
            font_color = font_color))
    file.close()
    if not os.path.exists("conf"):
        os.mkdir("conf")
    
    file = open("conf/highlight.conf", "w")
    file.write('''fill_color       = grey
color            = {font_color}
stroke_thickness = 1
r1               = 0.5r
r0               = 1r

<highlights>

<highlight>

#Positive band BGCs
init_counter = highlight:1
file = {folder}/{output}_CDS_pos.txt
fill_color = {CDS_positive}
r1 = 0.85r
r0 = 0.89r
</highlight>

#Negative
<highlight>

file = {folder}/{output}_CDS_neg.txt
fill_color = {CDS_negative}
r1 = 0.8r
r0 = 0.84r
</highlight>

#Positive band tRNAs
<highlight>
file = {folder}/{output}_tRNA_pos.txt
fill_color = {tRNA_positive}
r1 = 0.85r
r0 = 0.89r
</highlight>

#Negative
<highlight>
file = {folder}/{output}_tRNA_neg.txt
fill_color = {tRNA_negative}
r1 = 0.8r
r0 = 0.84r
</highlight>

#Positive band rRNAs
<highlight>
file = {folder}/{output}_rRNA_pos.txt
fill_color = {rRNA_positive}
r1 = 0.85r
r0 = 0.89r
</highlight>

#Negative
<highlight>
file = {folder}/{output}_rRNA_neg.txt
fill_color = {rRNA_negative}
r1 = 0.8r
r0 = 0.84r
</highlight>

'''.format(folder = folder,
           output = output,
           CDS_positive = CDS_positive_color,
           CDS_negative = CDS_negative_color,
           tRNA_positive = tRNA_color,
           tRNA_negative = tRNA_color,
           rRNA_positive = rRNA_color,
           rRNA_negative = rRNA_color,
           font_color = font_color))
    if cogs:
        for COG in [{"name": "D", "color": "99, 123, 183"},
                    {"name": "M", "color": "38, 89, 168"},
                    {"name": "N", "color": "100, 151, 176"},
                    {"name": "O", "color": "32, 132, 174"},
                    {"name": "T", "color": "66, 169, 179"},
                    {"name": "U", "color": "18, 105, 116"},
                    {"name": "V", "color": "97, 195, 166"},
                    {"name": "W", "color": "37, 151, 117"},
                    {"name": "Y", "color": "70, 185, 109"},
                    {"name": "Z", "color": "21, 121, 60"},

                    {"name": "A", "color": "231, 82, 81"},
                    {"name": "B", "color": "208, 36, 41"},
                    {"name": "J", "color": "166, 96, 167"},
                    {"name": "K", "color": "157, 63, 151"},
                    {"name": "L", "color": "133, 116, 181"},
                    {"name": "X", "color": "79, 72, 158"},
                    
                    {"name": "C", "color": "170, 211, 130"},
                    {"name": "E", "color": "125, 176, 64"},
                    {"name": "F", "color": "178, 179, 109"},
                    {"name": "G", "color": "143, 138, 47"},
                    {"name": "H", "color": "235, 188, 134"},
                    {"name": "I", "color": "175, 126, 53"},
                    {"name": "P", "color": "219, 136, 86"},
                    {"name": "Q", "color": "196, 100, 38"},
                    
                    {"name": "R", "color": "98, 98, 98"},
                    {"name": "S", "color": "144, 144, 144"},
                    {"name": "None", "color": "234, 234, 234"},
                    ]:
            if COG["name"] in cogs_p:
                file.write('''
#positive {name}
<highlight>

file = {folder}/{output}_CDS_pos_{name}.txt
fill_color = {color}
r1 = 0.9r
r0 = 0.94r
</highlight>
'''.format(folder = folder, output = output, name = COG["name"], color = COG["color"]))
            if COG["name"] in cogs_n:
                file.write('''
#negative {name}
<highlight>

file = {folder}/{output}_CDS_neg_{name}.txt
fill_color = {color}
r1 = 0.75r
r0 = 0.79r
</highlight>
'''.format(folder = folder, output = output, name = COG["name"], color = COG["color"]))
    file.write("</highlights>")
    file.close()

    file = open("conf/colors_fonts_patterns.conf", "w")
    file.write('''<colors>
<<include etc/colors.conf>>
</colors>

<fonts>
<<include etc/fonts.conf>>
</fonts>

<patterns>
<<include etc/patterns.conf>>
</patterns>
''')
    file.close()

    file = open("conf/housekeeping.conf", "w")
    file.write('''anglestep       = 0.5
minslicestep    = 10
beziersamples   = 40 # bezier curves are drawn piece-wise
                     # linear, with this many samples
debug           = no
warnings        = no
imagemap        = no
paranoid        = yes

units_ok        = bupr
units_nounit    = n

# \t  tab
# \s  any whitespace
file_delim = \s
# collapse adjacent whitespace 
# e.g. two spaces are treated as one, not as a missing field
file_delim_collapse = yes

# Record delimiter for parameter values that are lists, such as
# hs1:0.25;hs2:0.10. By default, both ; and , are accepted
#
# e.g. hs1:0.25,hs2:0.10
#      hs1:0.25;hs2:0.10
list_record_delim = \s*[;,]\s*
# Field delimiter specifies the assignment operator, e.g. 
list_field_delim  = \s*[:=]\s*

# Record delimiter for options in data files.
options_record_delim = [,;]
options_field_delim  = =

# Rule fields and other parameters accept var(VARIABLE) syntax
# to reference parameters of data points. By default, if
# VARIABLE does not exist, Circos quits with an error, unless
# the skip parameter below is set.
# 
# This feature is useful when you have data that don't always
# have the same options. For example,
# 
# chr1 10 20 a=10
# chr1 50 60 b=10
skip_missing_expression_vars = no

# In old versions, data point parameters were referenced using _NAME_
# syntax. This has been replaced with var(NAME). The _NAME_ syntax is
# deprecated (for example, it will break when dealing with fields like
# gene_a_1). If you must use it, set the parameter below.

legacy_underline_expression_syntax = no

# Magnification factor for text in SVG files.
svg_font_scale = 1.3

# super/sub script baseline shift (%)
sup_baseline_shift = 40
sub_baseline_shift = -40

# super/sub script font size (%)
sup_fontsize = 90
sub_fontsize = 90

# default font - pick one of the keys from <fonts> block
default_font   = default
# default font name is used for SVG files for cases where
# the font definition does not include a name
# see etc/fonts.conf for details
default_font_name  = Arial
default_font_color = {font_color}

# default color for cases when color is not specified
default_color  = {font_color}

<guides>
thickness      = 1
size           = 5
type           = outline
<object>
all            = no
ideogram       = no
ideogram_label = no
</object>
<color>
default = lblue
text    = red
</color>
</guides>

# Receive debug messages about actions
# 
# For a full list of available debug groups, see 
#   
#  http://www.circos.ca/documentation/tutorials/configuration/debugging/
#
# Common ones are:
#
# summary   - top level indications of what's happening (on by default)
# chrfilter - ideogram filtering (parsing 'chromosomes' parameter)
# conf      - configuration file
# io        - file I/O
# ideogram  - ideogram reporting
# layers    - specific plot z-layers
# rules     - dynamic rules
# color     - color allocation and definition
# timers    - some benchmark timings
# cache     - caches
# _all      - turn on all groups

debug_group = summary,output

# run length duration after which timing report is automatically
# generated at the end of the run
debug_auto_timer_report = 30

debug_word_separator = " "
debug_undef_text     = _undef_
debug_empty_text     = _emptylist_

# parameters passed to functions can be
# validated to check consistency. turn this
# off to speed things up
debug_validate       = yes

# Reformat numbers in debug output for consistency.
# If you have a lot of debug output (e.g. -debug_group _all)
# this will slow things considerably.
debug_output_tidy    = no

# pixel sub-sampling for text tracks
text_pixel_subsampling = 1
# array or span - use 'span' if applying snuggle refinement
text_snuggle_method    = array

# restrict names of parameters?
# if 'yes' then only certain named parameters are allowed within
# blocks and option fields for data
#
# if 'no' then you can define parameters with any name what-so-ever,
# useful if you wish to define states or labels for your data
#
# e.g. hs1 10 20 0.5 paired=yes,special=no,myvar=0.5
#
# ordinarily, 'paired', 'special' and 'myvar' would not be allowed
restrict_parameter_names = no

# Unless set to 'yes', parameter names will be converted to lowercase
case_sensitive_parameter_names = no

# Calculate track statistics? Gives access to 
# min 
# max 
# average 
# n - number of data points in track
# sd - standard deviation
# var - variance (sd^2)
#
# Once these are calculated, you can refer to the value using plot(STATISTIC)
# e.g.
# <rule>
# condition = plot(average) < 0.5
# ...
calculate_track_statistics = yes

# The location of configuration and data files will be guessed if 
# (a) configuration file is not specified
# (b) data file paths are relative
# Circos will look in the following locations, where 
# SCRIPTPATH is the location of the 'circos' script (e.g. /usr/local/bin) and
# CWD is the current directory (where the 'circos' command was executed).
# All paths under CWD will be scanned first, then under SCRIPTPATH.
#
# [CWD,SCRIPTPATH]/.
# [CWD,SCRIPTPATH]/..
# [CWD,SCRIPTPATH]/etc/
# [CWD,SCRIPTPATH]/../etc
# [CWD,SCRIPTPATH]/../../etc
# [CWD,SCRIPTPATH]/data
# [CWD,SCRIPTPATH]/../data
# [CWD,SCRIPTPATH]/../../data
#
# If you would like to prepend this list with custom directories for
# data files, enter them as a CSV list here
# data_path = /home/martink/circos-tutorials 

# If the cache is static, it will always be used and will not be updated
# unless it is deleted (use -color_cache_rebuild on the command line).
# Otherwise, the cache will be updated if 
#  - config file is newer than cache file
#  - list of colors in config file is different than in cache file
color_cache_static = yes
color_cache_file   = circos.colorlist
color_lists_use    = yes
# if the directory is not defined, then the system will guess a temporary
# directory compatible with your operating system (using File::Temp)
# color_cache_dir    = /tmp

# Make some functions faster. This should always be 'yes' unless you
# want things to run slowly or suspect deep issues.
memoize = yes

# This is a debugging flag and should be set to 'no' for regular use
quit_on_dump = yes

offsets = 0,0

# Maximum number of image and data elements. If these are exceeded,
# Circos will quit with an error. These values are arbitrary, but in
# my experience images with significantly more data points than this
# are uninterpretable.

max_ticks            = 5000
max_ideograms        = 200
max_links            = 25000
max_points_per_track = 25000

# What to do when data is found for an ideogram that does not appear in the karyotype file.
# Set to 'skip' or 'exit'
undefined_ideogram = skip

# Number of iterations for determining ideogram sizes when
# relative scale is used.
relative_scale_iterations = 10

# min, max, average, mode             - based on scale statistics of ALL ideograms
# minadj, maxadj, averageadj, modeadj - based on scale statistics of adjacent ideograms
# 
# You can specify a fixed scale for spacing using a floating value
#
# e.g. relative_scale_spacing = 1.5
relative_scale_spacing    = mode

# What to do with out-of-range data, or data that extends beyond
# ideogram range. This applies to all plot types and links.  Set to
# 'hide' or 'trim' or 'fatal'. For 'hide' and 'trim', optionally add
# 'warn' to produce a warning. Data points that fall entirely
# outside of their ideogram will never be shown, regardless whether
# you've set 'hide' or 'trim'.
#
# This check DOES NOT trigger for data that falls on ideograms that
# are not being displayed.
#
# e.g.
# hide
# hide,warn
# trim
# trim,warn
# fatal
data_out_of_range = trim,warn # 

# Track default directory
track_defaults = etc/tracks

# Use round brushes for elements with thickness greater than round_brush_min_thickness?
round_brush_use           = yes
round_brush_min_thickness = 5

# Use anti aliasing, where possible? I've seen bugs in some gd libraries
# that cause artefacts to appear when lines are anti-aliased. If your
# image contains unexpected elements, turn aa off.
anti_aliasing = yes

# A parameter that must be set. Checks whether this file was imported.
housekeeping = yes

# Should parameter values be evaluated as code by default? This
# allows you to avoid eval() in expressions. However, this is
# dangerous because it does not return errors.
auto_eval = no
'''.format(font_color = font_color))
    file.close()

    file = open("conf/image.conf", "w")
    file.write('''<<include image.generic.conf>>
background = {}
'''.format(background_color))
    file.close()

    file = open("conf/ticks.conf", "w")
    file.write('''# TICKS
show_ticks          = yes
show_tick_labels    = yes
<ticks>
skip_first_label     = no
skip_last_label      = no
radius           = dims(ideogram,radius_outer)
thickness        = 2p
tick_separation      = 2p
min_label_distance_to_edge = 0p
label_separation = 5p
label_offset     = 5p
multiplier = 0.000001
color = {font_color}
<tick>
size     = 7p
thickness      = 1p
spacing        = 1000u
show_label     = no
</tick>
<tick>
size     = 10p
thickness      = 3p
spacing        = 5000u
show_label     = yes
suffix = " Mb"
label_size     = 20p
#format         = %s
</tick>
</ticks>
'''.format(font_color = font_color))
    file.close()
    return

def create_conf_main():
    parser = ap.ArgumentParser()

    limit_group = parser.add_argument_group("limits")
    limit_group.add_argument("--content_min", "--min_GC_content", type=str, help="Minimum GC content", required=False, default = "0")
    limit_group.add_argument("--content_max", "--max_GC_content", type=str, help="Maximum GC content", required=False, default = "100")
    limit_group.add_argument("--skew_min", "--min_GC_skew", type=str, help="Minimum GC skew", required=False, default = "-1")
    limit_group.add_argument("--skew_max", "--max_GC_skew", type=str, help="Maximum GC skew", required=False, default = "1")

    color_group = parser.add_argument_group("colors")
    color_group.add_argument("-cc", "--GC_content_color", type=str, help="Color for GC content, in R, G, B format. Default: '23, 0, 115'", default = "23, 0, 115")
    color_group.add_argument("-sc", "--GC_skew_color", type=str, help="Color scheme for GC skew. For details on this, please read CIRCOS documentation. Default: 'eval(sprintf(\"rdbu-7-div-%%d\",remap_int(var(value),0,0,7,5)))'", default = 'eval(sprintf("rdbu-7-div-%d",remap_int(var(value),0,0,7,5)))')
    color_group.add_argument("-pc", "--CDS_positive_color", type=str, help="Color for positive CDSs, in R, G, B format. Default: '180, 205, 222'", default = '180, 205, 222')
    color_group.add_argument("-nc", "--CDS_negative_color", type=str, help="Color for negative CDSs, in R, G, B format. Default: '53, 176, 42'", default = '53, 176, 42')

    args = parser.parse_args()
    create_conf({"min_GC_content": args.min_GC_content,
                 "max_GC_content": args.max_GC_content,
                 "min_skew": args.min_GC_skew,
                 "max_skew": args.max_GC_skew},
                 args.GC_content_color,
                 args.GC_skew_color,
                 args.CDS_positive_color,
                 args.CDS_negative_color)

if __name__ == "__main__":
    parser = ap.ArgumentParser()

    limit_group = parser.add_argument_group("limits")
    limit_group.add_argument("--content_min", "--min_GC_content", type=str, help="Minimum GC content", required=False, default = "0")
    limit_group.add_argument("--content_max", "--max_GC_content", type=str, help="Maximum GC content", required=False, default = "100")
    limit_group.add_argument("--skew_min", "--min_GC_skew", type=str, help="Minimum GC skew", required=False, default = "-1")
    limit_group.add_argument("--skew_max", "--max_GC_skew", type=str, help="Maximum GC skew", required=False, default = "1")

    color_group = parser.add_argument_group("colors")
    color_group.add_argument("-cc", "--GC_content_color", type=str, help="Color for GC content, in R, G, B format. Default: '23, 0, 115'", default = "23, 0, 115")
    color_group.add_argument("-sc", "--GC_skew_color", type=str, help="Color scheme for GC skew. For details on this, please read CIRCOS documentation. Default: 'eval(sprintf(\"rdbu-7-div-%%d\",remap_int(var(value),0,0,7,5)))'", default = 'eval(sprintf("rdbu-7-div-%d",remap_int(var(value),0,0,7,5)))')
    color_group.add_argument("-pc", "--CDS_positive_color", type=str, help="Color for positive CDSs, in R, G, B format. Default: '180, 205, 222'", default = '180, 205, 222')
    color_group.add_argument("-nc", "--CDS_negative_color", type=str, help="Color for negative CDSs, in R, G, B format. Default: '53, 176, 42'", default = '53, 176, 42')

    args = parser.parse_args()
    create_conf({"min_GC_content": args.min_GC_content,
                 "max_GC_content": args.max_GC_content,
                 "min_skew": args.min_GC_skew,
                 "max_skew": args.max_GC_skew},
                 args.GC_content_color,
                 args.GC_skew_color,
                 args.CDS_positive_color,
                 args.CDS_negative_color)
